﻿Namespace System.Drawing

   ''' <summary>a compatibily-Structure for Size and Point</summary>
   <Serializable()> _
   <DebuggerDisplay("{X} {Y}")> _
   Public Structure Vector
      ' both Size and Point can convert to and from Vector.
      ' X,Y also can be accessed by Index 0 or 1 as well as by Index True or False. This avoids Code-
      '°Repeating, because you don't need to implement individual computation-code for horizontal and
      '°vertical purposes - you can use the **same** code, just passing another Index.

      Public Shared ReadOnly Property MinValue() As Vector
         Get
            Return New Vector(Integer.MinValue, Integer.MinValue)
         End Get
      End Property
      Public Shared ReadOnly Property Empty() As Vector
         Get
            Return New Vector(0, 0)
         End Get
      End Property

      Public X, Y As Integer
      Public Sub New( x As Integer,  y As Integer)
         Me.X = x : Me.Y = y
      End Sub
      Default Public Property Amount( i As Integer) As Integer
         Get
            If i = 0 Then Return X
            Return Y
         End Get
         Set( value As Integer)
            If i = 0 Then X = value Else Y = value
         End Set
      End Property
      Default Public Property Amount( vertical As Boolean) As Integer
         Get
            If vertical Then Return Y
            Return X
         End Get
         Set( value As Integer)
            If vertical Then Y = value Else X = value
         End Set
      End Property
      ''' <summary>
      ''' return-values: -1: no difference, 0: horizontal diff, 1: vertical diff, 2: diff in both directions
      ''' </summary>
      Public Function DifferenceDirection( v2 As Vector) As Integer
         Dim ret = 0
         For i = 0 To 1
            If Amount(i) <> v2(i) Then ret = ret Or i + 1
         Next
         Return ret - 1
      End Function
      Public Overrides Function ToString() As String
         Return String.Concat("{", X, " ", Y, "}")
      End Function
      Public Function Scale( x As Double,  y As Double) As Vector
         Return New Vector(CInt(Me.X * x), CInt(Me.Y * y))
      End Function
      Public Function Scale( value As Double) As Vector
         Return New Vector(CInt(Me.X * value), CInt(Me.Y * value))
      End Function

      Public Function ToPoint() As Point
         Return New Point(X, Y)
      End Function
      Public Function ToSize() As Size
         Return New Size(X, Y)
      End Function
      Public ReadOnly Property IsMinimum() As Boolean
         Get
            Return Me = MinValue
         End Get
      End Property
      Public ReadOnly Property NotMinimum() As Boolean
         Get
            Return Me <> MinValue
         End Get
      End Property
      Public ReadOnly Property IsEmpty() As Boolean
         Get
            Return Me = Empty
         End Get
      End Property
      Public ReadOnly Property NotEmpty() As Boolean
         Get
            Return Me <> Empty
         End Get
      End Property
      Public Function Union( other As Vector) As Vector
         Return New Vector(Math.Max(X, other.X), Math.Max(Y, other.Y))
      End Function
      Public Function Intersect( other As Vector) As Vector
         Return New Vector(Math.Min(X, other.X), Math.Min(Y, other.Y))
      End Function
      Public Shared Widening Operator CType( sz As Size) As Vector
         Return New Vector(sz.Width, sz.Height)
      End Operator
      Public Shared Widening Operator CType( pt As Point) As Vector
         Return New Vector(pt.X, pt.Y)
      End Operator
      Public Shared Widening Operator CType( v As Vector) As Point
         Return v.ToPoint
      End Operator
      Public Shared Widening Operator CType( v As Vector) As Size
         Return v.ToSize
      End Operator
      Public Shared Operator +( v1 As Rectangle,  v2 As Vector) As Rectangle
         v1.Location += v2
         Return v1
      End Operator
      Public Shared Operator +( v1 As Point,  v2 As Vector) As Point
         Return New Point(v1.X + v2.X, v1.Y + v2.Y)
      End Operator
      Public Shared Operator +( v1 As Vector,  v2 As Point) As Vector
         Return New Vector(v1.X + v2.X, v1.Y + v2.Y)
      End Operator
      Public Shared Operator +( v1 As Size,  v2 As Vector) As Size
         Return New Size(v1.Width + v2.X, v1.Height + v2.Y)
      End Operator
      Public Shared Operator +( v1 As Vector,  v2 As Vector) As Vector
         Return New Vector(v1.X + v2.X, v1.Y + v2.Y)
      End Operator
      Public Shared Operator *( v1 As Vector, n As Integer) As Vector
         Return New Vector(v1.X * n, v1.Y * n)
      End Operator
      Public Shared Operator \( v1 As Vector, n As Integer) As Vector
         Return New Vector(v1.X \ n, v1.Y \ n)
      End Operator
      Public Shared Operator +( v1 As Vector,  sz As Size) As Vector
         Return New Vector(v1.X + sz.Width, v1.Y + sz.Height)
      End Operator
      Public Shared Operator -( v2 As Vector) As Vector
         Return New Vector(-v2.X, -v2.Y)
      End Operator
      Public Shared Operator -( v1 As Size,  v2 As Vector) As Vector
         Return New Vector(v1.Width - v2.X, v1.Height - v2.Y)
      End Operator
      Public Shared Operator -( v1 As Vector,  v2 As Vector) As Vector
         Return New Vector(v1.X - v2.X, v1.Y - v2.Y)
      End Operator
      Public Shared Operator -( v1 As Vector,  sz As Size) As Vector
         Return New Vector(v1.X - sz.Width, v1.Y - sz.Height)
      End Operator
      Public Shared Operator =( v1 As Size,  v2 As Vector) As Boolean
         Return v1.Width = v2.X AndAlso v1.Height = v2.Y
      End Operator
      Public Shared Operator <>( v1 As Size,  v2 As Vector) As Boolean
         Return Not v1 = v2
      End Operator
      Public Shared Operator =( v1 As Vector,  v2 As Size) As Boolean
         Return v1.X = v2.Width AndAlso v1.Y = v2.Height
      End Operator
      Public Shared Operator <>( v1 As Vector,  v2 As Size) As Boolean
         Return Not v1 = v2
      End Operator
      Public Shared Operator =( v1 As Point,  v2 As Vector) As Boolean
         Return v1.X = v2.X AndAlso v1.Y = v2.Y
      End Operator
      Public Shared Operator <>( v1 As Point,  v2 As Vector) As Boolean
         Return Not v1 = v2
      End Operator
      Public Shared Operator =( v1 As Vector,  v2 As Point) As Boolean
         Return v1.X = v2.X AndAlso v1.Y = v2.Y
      End Operator
      Public Shared Operator <>( v1 As Vector,  v2 As Point) As Boolean
         Return Not v1 = v2
      End Operator
      Public Shared Operator =( v1 As Vector,  v2 As Vector) As Boolean
         Return v1.X = v2.X AndAlso v1.Y = v2.Y
      End Operator
      Public Shared Operator <>( v1 As Vector,  v2 As Vector) As Boolean
         Return Not v1 = v2
      End Operator
   End Structure

End Namespace
